package bookExamples.ch26Graphics.draw2d.shapes;


import bookExamples.ch26Graphics.draw2d.Containment;
import bookExamples.ch26Graphics.draw2d.Intersects;
import bookExamples.ch26Graphics.draw2d.Ray2d;
import bookExamples.ch26Graphics.draw2d.Vec2d;

import java.awt.*;
import java.beans.PropertyChangeListener;
import java.util.Vector;

public class Shapes extends DJShape implements Containment {

    // Number of times ray reflects before stopping
    private final static int maxCount = 3;

    // Refractive index
    private final static double refIndex = 1.2;

    private Circle2d closestCircle;

    // todo compute the aggregate venterV2d;
    public Point getCenter() {
        return new Point(0, 0);
    }

    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new Shapes(x1, y1, x2, y2);
    }

    private Vector v = new Vector();

    public Shapes(int x1, int y1, int x2, int y2) {
        super(x1, y1, x2, y2);
    }

    public ImageShape removeImageShape() {
        for (int i = 0; i < getV().size(); i++) {
            Object o = getV().elementAt(i);
            if (o instanceof ImageShape) {
                getV().removeElementAt(i);
                return (ImageShape) o;
            }
        }
        return null;
    }

    public ImageShape getImageShape() {
        for (int i = 0; i < getV().size(); i++) {
            Object o = getV().elementAt(i);
            if (o instanceof ImageShape) {
                return (ImageShape) o;
            }
        }
        return null;
    }

    public void addShape(DJShape d) {
        if (d == null) {
            System.out.println("d==null!!");
        }

        getV().addElement(d);
    }

    public void draw(Graphics g) {
        for (int i = 0; i < getV().size(); i++) {
            DJShape d =
                    (DJShape) getV().elementAt(i);
            d.draw(g);
        }
    }

    public Line2d getLastLine() {
        for (int i = getV().size() - 1; i >= 0; i--)
            if (getV().elementAt(i) instanceof Line2d)
                return
                        (Line2d) getV().elementAt(i);
        return null;
    }

    public DJShape removeLastShape() {
        DJShape js = (DJShape) (getV().elementAt(getV().size() - 1));
        getV().remove(js);
        return js;
    }

    public void setLastShape(DJShape js) {
        getV().setElementAt(js, getV().size() - 1);
    }

    public Circle2d getLastCircle() {
        for (int i = getV().size() - 1; i >= 0; i--)
            if (getV().elementAt(i) instanceof Circle2d)
                return
                        (Circle2d) getV().elementAt(i);
        return null;
    }

    public Rect2d getLastRectangle() {
        for (int i = getV().size() - 1; i >= 0; i--)
            if (v.elementAt(i) instanceof Rect2d)
                return
                        (Rect2d) getV().elementAt(i);
        return null;
    }

    public Vec2d getClosestPoint(Ray2d ray) {
        double maxDist = 1000000;
        Vec2d closestSoFar = null;

        for (int i = getV().size() - 1; i >= 0; i--) {
            if (!(getV().elementAt(i) instanceof Intersects))
                continue;
            Intersects inter = (Intersects) getV().elementAt(i);
            Vec2d vc = inter.intersect(ray);
            if (vc == null) continue;
            if (ray.getT() >= maxDist) continue;
            closestSoFar = vc;
            if (getV().elementAt(i) instanceof Circle2d) {
                setClosestCircle((Circle2d) v.elementAt(i));
            } else {
                setClosestCircle(null);
            }
            maxDist = ray.getT();
        }
        return closestSoFar;
    }

    public void addLineToLastCircle() {

        Line2d l = getLastLine();
        if (l == null) return;
        Ray2d r = new Ray2d(l);
        addLineToLastCircle(r);
    }

    public void addLineToLastCircle(Ray2d r) {

        Vec2d v = getClosestPoint(r);
        if (v == null) return;

        if (getClosestCircle() != null) {

            Ray2d rl = getReflectRay(getClosestCircle(), v, r.getDirection());
            Ray2d rf = getRefractRay(getClosestCircle(), r.getOrigin(), v, r.getDirection());

            rl.getOrigin().add(rl.getDirection());
            rf.getOrigin().add(rf.getDirection());

            addShape(new Line2d(rl.getOrigin(), rl.vecOnLine(50)));
            addShape(new Line2d(rf.getOrigin(), rf.vecOnLine(50)));

            if (r.getCount() < getMaxCount()) {
                rl.setCount(r.getCount() + 1);
                addLineToLastCircle(rl);
            }
            if (r.getCount() < getMaxCount()) {
                rf.setCount(r.getCount() + 1);
                addLineToLastCircle(rf);
            }
        }
        addShape(new Line2d(r.getOrigin(), v));
    }

    public Ray2d getReflectRay(Circle2d c, Vec2d p, Vec2d d) {

        Vec2d n = new Vec2d(p);
        n.sub(c.getVenterV2d());
        n.normalize();

        n.mult(2 * d.dot(n));
        d.sub(n);
        return new Ray2d(p, d);
    }

    public Ray2d getRefractRay(Circle2d c, Vec2d o, Vec2d p, Vec2d d) {

        double t;
        double idx;

        Vec2d n = new Vec2d(p);
        n.sub(c.getVenterV2d());
        n.normalize();


        if (c.inside(o)) {
            idx = 1 / getRefIndex();
            n.mult(-1);
        } else {
            idx = getRefIndex();
        }


        t = 1 / d.dot(n);

        Vec2d u = new Vec2d(d);
        Vec2d v = new Vec2d(n);

        u.mult(t);
        v.add(u);


        t = 1 / ((idx * idx) * u.dot(u) - v.dot(v));

        u.mult(t);
        u.sub(n);
        n.mult(t);
        u.add(n);
        u.normalize();

        return new Ray2d(p, u);

    }

    public DJShape getClosestShape(Point p) {
        double minDistance = Double.MAX_VALUE;
        DJShape closestDJShape = null;
        for (int i = 0; i < getV().size(); i++) {
            DJShape s = (DJShape) getV().elementAt(i);
            double distance = s.getCenterPoint().distance(p);
            if (minDistance > distance) {
                minDistance = distance;
                closestDJShape = s;
            }
        }
        return closestDJShape;
    }

    public boolean isPaintable() {
        return false;
    }

    public boolean supportsCustomEditor() {
        return false;
    }

    public Component getCustomEditor() {
        return null;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
    }

    public Object getValue() {
        return null;
    }

    public void setValue(Object value) {
    }

    public String getAsText() {
        return null;
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String[] getTags() {
        return new String[0];
    }

    public void setAsText(String text) throws IllegalArgumentException {
    }

    public void paintValue(Graphics gfx, Rectangle box) {
    }

    public void eraseLastShape(DJShape shape) {
        getV().remove(shape);
    }


    public boolean contains(double x, double y) {
        for (int i = 0; i < getV().size(); i++) {
            DJShape s = (DJShape) getV().elementAt(i);
            if (! (s instanceof Containment)) 
                continue;
            Containment c = (Containment) s;
            if (c.contains(x, y)) return true;
        }
        return false;
    }

    public static int getMaxCount() {
        return maxCount;
    }

    public static double getRefIndex() {
        return refIndex;
    }

    public Circle2d getClosestCircle() {
        return closestCircle;
    }

    public void setClosestCircle(Circle2d closestCircle) {
        this.closestCircle = closestCircle;
    }

    public Vector getV() {
        return v;
    }

    public void setV(Vector v) {
        this.v = v;
    }
}
